@@ -17,9 +17,11 @@ limitations under the License.
 package mockelbv2
 
 import (
+	"context"
 	"fmt"
 
 	"github.com/aws/aws-sdk-go/aws"
+	"github.com/aws/aws-sdk-go/aws/request"
 	"github.com/aws/aws-sdk-go/service/ec2"
 	"github.com/aws/aws-sdk-go/service/elbv2"
 	"k8s.io/klog/v2"
@@ -74,6 +76,10 @@ func (m *MockELBV2) DescribeLoadBalancersPages(request *elbv2.DescribeLoadBalanc
 	return nil
 }
 
+func (m *MockELBV2) DescribeLoadBalancersPagesWithContext(ctx context.Context, request *elbv2.DescribeLoadBalancersInput, callback func(p *elbv2.DescribeLoadBalancersOutput, lastPage bool) (shouldContinue bool), opts ...request.Option) error {
+	return m.DescribeLoadBalancersPages(request, callback)
+}
+
 func (m *MockELBV2) CreateLoadBalancer(request *elbv2.CreateLoadBalancerInput) (*elbv2.CreateLoadBalancerOutput, error) {
 	m.mutex.Lock()
 	defer m.mutex.Unlock()
@@ -83,6 +89,7 @@ func (m *MockELBV2) CreateLoadBalancer(request *elbv2.CreateLoadBalancerInput) (
 	lb := elbv2.LoadBalancer{
 		LoadBalancerName:      request.Name,
 		Scheme:                request.Scheme,
+		SecurityGroups:        request.SecurityGroups,
 		Type:                  request.Type,
 		IpAddressType:         request.IpAddressType,
 		DNSName:               aws.String(fmt.Sprintf("%v.amazonaws.com", aws.StringValue(request.Name))),
@@ -194,6 +201,19 @@ func (m *MockELBV2) ModifyLoadBalancerAttributes(request *elbv2.ModifyLoadBalanc
 	return nil, fmt.Errorf("LoadBalancerNotFound: %v", aws.StringValue(request.LoadBalancerArn))
 }
 
+func (m *MockELBV2) SetSecurityGroups(request *elbv2.SetSecurityGroupsInput) (*elbv2.SetSecurityGroupsOutput, error) {
+	m.mutex.Lock()
+	defer m.mutex.Unlock()
+	arn := aws.StringValue(request.LoadBalancerArn)
+	if lb, ok := m.LoadBalancers[arn]; ok {
+		lb.description.SecurityGroups = request.SecurityGroups
+		return &elbv2.SetSecurityGroupsOutput{
+			SecurityGroupIds: request.SecurityGroups,
+		}, nil
+	}
+	return nil, fmt.Errorf("LoadBalancerNotFound: %v", aws.StringValue(request.LoadBalancerArn))
+}
+
 func (m *MockELBV2) SetSubnets(request *elbv2.SetSubnetsInput) (*elbv2.SetSubnetsOutput, error) {
 	m.mutex.Lock()
 	defer m.mutex.Unlock()
