@@ -32,7 +32,9 @@ import (
 	"k8s.io/kops/cmd/kops-controller/controllers"
 	"k8s.io/kops/cmd/kops-controller/pkg/config"
 	"k8s.io/kops/cmd/kops-controller/pkg/server"
+	"k8s.io/kops/pkg/apis/kops/v1alpha2"
 	"k8s.io/kops/pkg/bootstrap"
+	"k8s.io/kops/pkg/bootstrap/pkibootstrap"
 	"k8s.io/kops/pkg/nodeidentity"
 	nodeidentityaws "k8s.io/kops/pkg/nodeidentity/aws"
 	nodeidentityazure "k8s.io/kops/pkg/nodeidentity/azure"
@@ -58,7 +60,6 @@ import (
 )
 
 var (
-	scheme   = runtime.NewScheme()
 	setupLog = ctrl.Log.WithName("setup")
 )
 
@@ -100,12 +101,16 @@ func main() {
 
 	ctrl.SetLogger(klogr.New())
 
-	if err := buildScheme(); err != nil {
+	scheme, err := buildScheme()
+	if err != nil {
 		setupLog.Error(err, "error building scheme")
 		os.Exit(1)
 	}
 
 	kubeConfig := ctrl.GetConfigOrDie()
+	kubeConfig.Burst = 200
+	kubeConfig.QPS = 100
+
 	mgr, err := ctrl.NewManager(kubeConfig, ctrl.Options{
 		Scheme: scheme,
 		Metrics: metricsserver.Options{
@@ -122,52 +127,76 @@ func main() {
 	vfsContext := vfs.NewVFSContext()
 
 	if opt.Server != nil {
-		var verifier bootstrap.Verifier
+		var verifiers []bootstrap.Verifier
 		var err error
 		if opt.Server.Provider.AWS != nil {
-			verifier, err = awsup.NewAWSVerifier(opt.Server.Provider.AWS)
+			verifier, err := awsup.NewAWSVerifier(opt.Server.Provider.AWS)
 			if err != nil {
 				setupLog.Error(err, "unable to create verifier")
 				os.Exit(1)
 			}
-		} else if opt.Server.Provider.GCE != nil {
-			verifier, err = gcetpmverifier.NewTPMVerifier(opt.Server.Provider.GCE)
+			verifiers = append(verifiers, verifier)
+		}
+		if opt.Server.Provider.GCE != nil {
+			verifier, err := gcetpmverifier.NewTPMVerifier(opt.Server.Provider.GCE)
 			if err != nil {
 				setupLog.Error(err, "unable to create verifier")
 				os.Exit(1)
 			}
-		} else if opt.Server.Provider.Hetzner != nil {
-			verifier, err = hetzner.NewHetznerVerifier(opt.Server.Provider.Hetzner)
+			verifiers = append(verifiers, verifier)
+		}
+		if opt.Server.Provider.Hetzner != nil {
+			verifier, err := hetzner.NewHetznerVerifier(opt.Server.Provider.Hetzner)
 			if err != nil {
 				setupLog.Error(err, "unable to create verifier")
 				os.Exit(1)
 			}
-		} else if opt.Server.Provider.OpenStack != nil {
-			verifier, err = openstack.NewOpenstackVerifier(opt.Server.Provider.OpenStack)
+			verifiers = append(verifiers, verifier)
+		}
+		if opt.Server.Provider.OpenStack != nil {
+			verifier, err := openstack.NewOpenstackVerifier(opt.Server.Provider.OpenStack)
 			if err != nil {
 				setupLog.Error(err, "unable to create verifier")
 				os.Exit(1)
 			}
-		} else if opt.Server.Provider.DigitalOcean != nil {
-			verifier, err = do.NewVerifier(ctx, opt.Server.Provider.DigitalOcean)
+			verifiers = append(verifiers, verifier)
+		}
+		if opt.Server.Provider.DigitalOcean != nil {
+			verifier, err := do.NewVerifier(ctx, opt.Server.Provider.DigitalOcean)
 			if err != nil {
 				setupLog.Error(err, "unable to create verifier")
 				os.Exit(1)
 			}
-		} else if opt.Server.Provider.Scaleway != nil {
-			verifier, err = scaleway.NewScalewayVerifier(ctx, opt.Server.Provider.Scaleway)
+			verifiers = append(verifiers, verifier)
+		}
+		if opt.Server.Provider.Scaleway != nil {
+			verifier, err := scaleway.NewScalewayVerifier(ctx, opt.Server.Provider.Scaleway)
 			if err != nil {
 				setupLog.Error(err, "unable to create verifier")
 				os.Exit(1)
 			}
-		} else if opt.Server.Provider.Azure != nil {
-			verifier, err = azure.NewAzureVerifier(ctx, opt.Server.Provider.Azure)
+			verifiers = append(verifiers, verifier)
+		}
+		if opt.Server.Provider.Azure != nil {
+			verifier, err := azure.NewAzureVerifier(ctx, opt.Server.Provider.Azure)
 			if err != nil {
 				setupLog.Error(err, "unable to create verifier")
 				os.Exit(1)
 			}
-		} else {
-			klog.Fatalf("server cloud provider config not provided")
+			verifiers = append(verifiers, verifier)
+		}
+
+		if opt.Server.PKI != nil {
+			verifier, err := pkibootstrap.NewVerifier(opt.Server.PKI, mgr.GetClient())
+			if err != nil {
+				setupLog.Error(err, "unable to create verifier")
+				os.Exit(1)
+			}
+			verifiers = append(verifiers, verifier)
+		}
+
+		if len(verifiers) == 0 {
+			klog.Fatalf("server verifiers not provided")
 		}
 
 		uncachedClient, err := client.New(mgr.GetConfig(), client.Options{
@@ -179,6 +208,8 @@ func main() {
 			os.Exit(1)
 		}
 
+		verifier := bootstrap.NewChainVerifier(verifiers...)
+
 		srv, err := server.NewServer(vfsContext, &opt, verifier, uncachedClient)
 		if err != nil {
 			setupLog.Error(err, "unable to create server")
@@ -214,15 +245,19 @@ func main() {
 	}
 }
 
-func buildScheme() error {
+func buildScheme() (*runtime.Scheme, error) {
+	scheme := runtime.NewScheme()
 	if err := corev1.AddToScheme(scheme); err != nil {
-		return fmt.Errorf("error registering corev1: %v", err)
+		return nil, fmt.Errorf("error registering corev1: %v", err)
+	}
+	if err := v1alpha2.AddToScheme(scheme); err != nil {
+		return nil, fmt.Errorf("error registering kops/v1alpha2 API: %v", err)
 	}
 	// Needed so that the leader-election system can post events
 	if err := coordinationv1.AddToScheme(scheme); err != nil {
-		return fmt.Errorf("error registering coordinationv1: %v", err)
+		return nil, fmt.Errorf("error registering coordinationv1: %v", err)
 	}
-	return nil
+	return scheme, nil
 }
 
 func addNodeController(mgr manager.Manager, vfsContext *vfs.VFSContext, opt *config.Options) error {
