@@ -28,10 +28,3 @@ func TestContainerdHooksBuilder(t *testing.T) {
 		return builder.Build(target)
 	})
 }
-
-func TestDockerHooksBuilder(t *testing.T) {
-	RunGoldenTest(t, "tests/golden/hooks-docker-exec", "hooks", func(nodeupModelContext *NodeupModelContext, target *fi.NodeupModelBuilderContext) error {
-		builder := HookBuilder{NodeupModelContext: nodeupModelContext}
-		return builder.Build(target)
-	})
-}
