@@ -72,7 +72,19 @@ func (b *KubeletBuilder) Build(c *fi.NodeupModelBuilderContext) error {
 	}
 
 	{
-		t, err := buildKubeletComponentConfig(kubeletConfig)
+		// Set the provider ID to help speed node registration on large clusters
+		var providerID string
+		if b.CloudProvider() == kops.CloudProviderAWS {
+			sess := session.Must(session.NewSession())
+			metadata := ec2metadata.New(sess)
+			instanceIdentity, err := metadata.GetInstanceIdentityDocument()
+			if err != nil {
+				return err
+			}
+			providerID = fmt.Sprintf("aws:///%s/%s", instanceIdentity.AvailabilityZone, instanceIdentity.InstanceID)
+		}
+
+		t, err := buildKubeletComponentConfig(kubeletConfig, providerID)
 		if err != nil {
 			return err
 		}
@@ -95,7 +107,7 @@ func (b *KubeletBuilder) Build(c *fi.NodeupModelBuilderContext) error {
 		// @TODO make Find call to an interface, we cannot mock out this function because it finds a file on disk
 		asset, err := b.Assets.Find(assetName, assetPath)
 		if err != nil {
-			return fmt.Errorf("error trying to locate asset %q: %v", assetName, err)
+			return fmt.Errorf("trying to locate asset %q: %v", assetName, err)
 		}
 		if asset == nil {
 			return fmt.Errorf("unable to locate asset %q", assetName)
@@ -157,13 +169,20 @@ func (b *KubeletBuilder) Build(c *fi.NodeupModelBuilderContext) error {
 		return err
 	}
 
-	if b.UseExternalECRCredentialsProvider() {
-		if err := b.addECRCP(c); err != nil {
-			return fmt.Errorf("failed to add ECR credential provider: %w", err)
+	if b.UseExternalKubeletCredentialProvider() {
+		switch b.CloudProvider() {
+		case kops.CloudProviderGCE:
+			if err := b.addGCPCredentialProvider(c); err != nil {
+				return fmt.Errorf("failed to add the %s kubelet credential provider: %w", b.CloudProvider(), err)
+			}
+		case kops.CloudProviderAWS:
+			if err := b.addECRCredentialProvider(c); err != nil {
+				return fmt.Errorf("failed to add the %s kubelet credential provider: %w", b.CloudProvider(), err)
+			}
 		}
 	}
 
-	if kubeletConfig.CgroupDriver == "systemd" && b.NodeupConfig.ContainerRuntime == "containerd" {
+	if kubeletConfig.CgroupDriver == "systemd" {
 
 		{
 			cgroup := kubeletConfig.KubeletCgroups
@@ -211,8 +230,11 @@ func (b *KubeletBuilder) Build(c *fi.NodeupModelBuilderContext) error {
 	return nil
 }
 
-func buildKubeletComponentConfig(kubeletConfig *kops.KubeletConfigSpec) (*nodetasks.File, error) {
+func buildKubeletComponentConfig(kubeletConfig *kops.KubeletConfigSpec, providerID string) (*nodetasks.File, error) {
 	componentConfig := kubelet.KubeletConfiguration{}
+	if providerID != "" {
+		componentConfig.ProviderID = providerID
+	}
 	if kubeletConfig.ShutdownGracePeriod != nil {
 		componentConfig.ShutdownGracePeriod = *kubeletConfig.ShutdownGracePeriod
 	}
@@ -264,11 +286,16 @@ func (b *KubeletBuilder) kubeletPath() string {
 	return b.binaryPath() + "/kubelet"
 }
 
-// ecrcpPath returns the path of the ECR credentials provider based on distro and archiecture
-func (b *KubeletBuilder) ecrcpPath() string {
+// getECRCredentialProviderPath returns the path of the ECR Credentials Provider based on distro and archiecture
+func (b *KubeletBuilder) getECRCredentialProviderPath() string {
 	return b.binaryPath() + "/ecr-credential-provider"
 }
 
+// getGCPCredentialProviderPath returns the path of the GCP Credentials Provider based on distro and archiecture
+func (b *KubeletBuilder) getGCPCredentialProviderPath() string {
+	return b.binaryPath() + "/gcp-credential-provider"
+}
+
 // buildManifestDirectory creates the directory where kubelet expects static manifests to reside
 func (b *KubeletBuilder) buildManifestDirectory(kubeletConfig *kops.KubeletConfigSpec) (*nodetasks.File, error) {
 	if kubeletConfig.PodManifestPath == "" {
@@ -311,23 +338,11 @@ func (b *KubeletBuilder) buildSystemdEnvironmentFile(kubeletConfig *kops.Kubelet
 	}
 
 	// Add container runtime spcific flags
-	switch b.NodeupConfig.ContainerRuntime {
-	case "docker":
-		if b.IsKubernetesLT("1.24") {
-			flags += " --container-runtime=docker"
-			flags += " --cni-bin-dir=" + b.CNIBinDir()
-			flags += " --cni-conf-dir=" + b.CNIConfDir()
-		}
-	case "containerd":
-		if b.IsKubernetesLT("1.24") {
-			flags += " --container-runtime=remote"
-		}
-		flags += " --runtime-request-timeout=15m"
-		if b.NodeupConfig.ContainerdConfig.Address == nil {
-			flags += " --container-runtime-endpoint=unix:///run/containerd/containerd.sock"
-		} else {
-			flags += " --container-runtime-endpoint=unix://" + fi.ValueOf(b.NodeupConfig.ContainerdConfig.Address)
-		}
+	flags += " --runtime-request-timeout=15m"
+	if b.NodeupConfig.ContainerdConfig.Address == nil {
+		flags += " --container-runtime-endpoint=unix:///run/containerd/containerd.sock"
+	} else {
+		flags += " --container-runtime-endpoint=unix://" + fi.ValueOf(b.NodeupConfig.ContainerdConfig.Address)
 	}
 
 	flags += " --tls-cert-file=" + b.PathSrvKubernetes() + "/kubelet-server.crt"
@@ -339,7 +354,7 @@ func (b *KubeletBuilder) buildSystemdEnvironmentFile(kubeletConfig *kops.Kubelet
 
 	flags += " --config=" + kubeletConfigFilePath
 
-	if b.UseExternalECRCredentialsProvider() {
+	if b.UseExternalKubeletCredentialProvider() {
 		flags += " --image-credential-provider-config=" + credentialProviderConfigFilePath
 		flags += " --image-credential-provider-bin-dir=" + b.binaryPath()
 	}
@@ -364,14 +379,7 @@ func (b *KubeletBuilder) buildSystemdService() *nodetasks.Service {
 	manifest := &systemd.Manifest{}
 	manifest.Set("Unit", "Description", "Kubernetes Kubelet Server")
 	manifest.Set("Unit", "Documentation", "https://github.com/kubernetes/kubernetes")
-	switch b.NodeupConfig.ContainerRuntime {
-	case "docker":
-		manifest.Set("Unit", "After", "docker.service")
-	case "containerd":
-		manifest.Set("Unit", "After", "containerd.service")
-	default:
-		klog.Warningf("unknown container runtime %q", b.NodeupConfig.ContainerRuntime)
-	}
+	manifest.Set("Unit", "After", "containerd.service")
 
 	manifest.Set("Service", "EnvironmentFile", "/etc/sysconfig/kubelet")
 
@@ -386,7 +394,7 @@ func (b *KubeletBuilder) buildSystemdService() *nodetasks.Service {
 
 	manifest.Set("Install", "WantedBy", "multi-user.target")
 
-	if b.NodeupConfig.KubeletConfig.CgroupDriver == "systemd" && b.NodeupConfig.ContainerRuntime == "containerd" {
+	if b.NodeupConfig.KubeletConfig.CgroupDriver == "systemd" {
 		cgroup := b.NodeupConfig.KubeletConfig.KubeletCgroups
 		if cgroup != "" {
 			manifest.Set("Service", "Slice", strings.Trim(cgroup, "/")+".slice")
@@ -421,21 +429,21 @@ func (b *KubeletBuilder) usesContainerizedMounter() bool {
 	}
 }
 
-// addECRCP installs the ECR credential provider
-func (b *KubeletBuilder) addECRCP(c *fi.NodeupModelBuilderContext) error {
+// addECRCredentialProvider installs the ECR Kubelet Credential Provider
+func (b *KubeletBuilder) addECRCredentialProvider(c *fi.NodeupModelBuilderContext) error {
 	{
 		assetName := "ecr-credential-provider-linux-" + string(b.Architecture)
 		assetPath := ""
 		asset, err := b.Assets.Find(assetName, assetPath)
 		if err != nil {
-			return fmt.Errorf("error trying to locate asset %q: %v", assetName, err)
+			return fmt.Errorf("trying to locate asset %q: %v", assetName, err)
 		}
 		if asset == nil {
 			return fmt.Errorf("unable to locate asset %q", assetName)
 		}
 
 		t := &nodetasks.File{
-			Path:     b.ecrcpPath(),
+			Path:     b.getECRCredentialProviderPath(),
 			Contents: asset,
 			Type:     nodetasks.FileType_File,
 			Mode:     s("0755"),
@@ -471,6 +479,56 @@ providers:
 	return nil
 }
 
+// addGCPCredentialProvider installs the GCP Kubelet Credential Provider
+func (b *KubeletBuilder) addGCPCredentialProvider(c *fi.NodeupModelBuilderContext) error {
+	{
+		assetName := "v20231005-providersv0.27.1-65-g8fbe8d27"
+		assetPath := ""
+		asset, err := b.Assets.Find(assetName, assetPath)
+		if err != nil {
+			return fmt.Errorf("trying to locate asset %q: %v", assetName, err)
+		}
+		if asset == nil {
+			return fmt.Errorf("unable to locate asset %q", assetName)
+		}
+
+		t := &nodetasks.File{
+			Path:     b.getGCPCredentialProviderPath(),
+			Contents: asset,
+			Type:     nodetasks.FileType_File,
+			Mode:     s("0755"),
+		}
+		c.AddTask(t)
+	}
+
+	{
+		configContent := `apiVersion: kubelet.config.k8s.io/v1
+kind: CredentialProviderConfig
+providers:
+  - apiVersion: credentialprovider.kubelet.k8s.io/v1
+    name: gcp-credential-provider
+    matchImages:
+      - "gcr.io"
+      - "*.gcr.io"
+      - "container.cloud.google.com"
+      - "*.pkg.dev"
+    defaultCacheDuration: "1m"
+    args:
+      - get-credentials
+      - --v=3
+`
+
+		t := &nodetasks.File{
+			Path:     credentialProviderConfigFilePath,
+			Contents: fi.NewStringResource(configContent),
+			Type:     nodetasks.FileType_File,
+			Mode:     s("0644"),
+		}
+		c.AddTask(t)
+	}
+	return nil
+}
+
 // addContainerizedMounter downloads and installs the containerized mounter, that we need on ContainerOS
 func (b *KubeletBuilder) addContainerizedMounter(c *fi.NodeupModelBuilderContext) error {
 	if !b.usesContainerizedMounter() {
@@ -490,7 +548,7 @@ func (b *KubeletBuilder) addContainerizedMounter(c *fi.NodeupModelBuilderContext
 		assetPath := ""
 		asset, err := b.Assets.Find(assetName, assetPath)
 		if err != nil {
-			return fmt.Errorf("error trying to locate asset %q: %v", assetName, err)
+			return fmt.Errorf("trying to locate asset %q: %v", assetName, err)
 		}
 		if asset == nil {
 			return fmt.Errorf("unable to locate asset %q", assetName)
@@ -513,7 +571,7 @@ func (b *KubeletBuilder) addContainerizedMounter(c *fi.NodeupModelBuilderContext
 	// TODO: leverage assets for this tar file (but we want to avoid expansion of the archive)
 	c.AddTask(&nodetasks.Archive{
 		Name:      "containerized_mounter",
-		Source:    "https://dl.k8s.io/gci-mounter/mounter.tar",
+		Source:    "https://storage.googleapis.com/kubernetes-release/gci-mounter/mounter.tar",
 		Hash:      "6a9f5f52e0b066183e6b90a3820b8c2c660d30f6ac7aeafb5064355bf0a5b6dd",
 		TargetDir: path.Join(containerizedMounterHome, "rootfs"),
 	})
