@@ -29,11 +29,10 @@ import (
 	"k8s.io/kops/upup/pkg/fi/cloudup/awsup"
 
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	v1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/kops/pkg/apis/kops"
 )
 
-func TestAWSValidateExternalCloudConfig(t *testing.T) {
+func TestAWSValidateEBSCSIDriver(t *testing.T) {
 	grid := []struct {
 		Input          kops.ClusterSpec
 		ExpectedErrors []string
@@ -49,7 +48,7 @@ func TestAWSValidateExternalCloudConfig(t *testing.T) {
 					},
 				},
 			},
-			ExpectedErrors: []string{"Forbidden::spec.externalCloudControllerManager"},
+			ExpectedErrors: []string{"Forbidden::spec.cloudProvider.aws.ebsCSIDriver.enabled"},
 		},
 		{
 			Input: kops.ClusterSpec{
@@ -80,7 +79,7 @@ func TestAWSValidateExternalCloudConfig(t *testing.T) {
 		cluster := &kops.Cluster{
 			Spec: g.Input,
 		}
-		errs := awsValidateExternalCloudControllerManager(cluster)
+		errs := awsValidateEBSCSIDriver(cluster)
 
 		testErrors(t, g.Input, errs, g.ExpectedErrors)
 	}
@@ -250,7 +249,7 @@ func TestValidateInstanceGroupSpec(t *testing.T) {
 
 	for _, g := range grid {
 		ig := &kops.InstanceGroup{
-			ObjectMeta: v1.ObjectMeta{
+			ObjectMeta: metav1.ObjectMeta{
 				Name: "test-nodes",
 			},
 			Spec: g.Input,
@@ -351,7 +350,7 @@ func TestMixedInstancePolicies(t *testing.T) {
 
 	for _, g := range grid {
 		ig := &kops.InstanceGroup{
-			ObjectMeta: v1.ObjectMeta{
+			ObjectMeta: metav1.ObjectMeta{
 				Name: "test-nodes",
 			},
 			Spec: g.Input,
@@ -383,7 +382,7 @@ func TestInstanceMetadataOptions(t *testing.T) {
 	}{
 		{
 			ig: &kops.InstanceGroup{
-				ObjectMeta: v1.ObjectMeta{
+				ObjectMeta: metav1.ObjectMeta{
 					Name: "some-ig",
 				},
 				Spec: kops.InstanceGroupSpec{
@@ -399,7 +398,7 @@ func TestInstanceMetadataOptions(t *testing.T) {
 		},
 		{
 			ig: &kops.InstanceGroup{
-				ObjectMeta: v1.ObjectMeta{
+				ObjectMeta: metav1.ObjectMeta{
 					Name: "some-ig",
 				},
 				Spec: kops.InstanceGroupSpec{
@@ -585,6 +584,9 @@ func TestLoadBalancerSubnets(t *testing.T) {
 						Type:  kops.LoadBalancerTypeInternal,
 					},
 				},
+				CloudProvider: kops.CloudProviderSpec{
+					AWS: &kops.AWSSpec{},
+				},
 			},
 		}
 		if test.class != nil {
@@ -668,6 +670,9 @@ func TestAWSAuthentication(t *testing.T) {
 						IdentityMappings: test.identityMappings,
 					},
 				},
+				CloudProvider: kops.CloudProviderSpec{
+					AWS: &kops.AWSSpec{},
+				},
 			},
 		}
 		errs := awsValidateCluster(&cluster, true)
