@@ -43,20 +43,22 @@ func (b *ClusterAutoscalerOptionsBuilder) BuildOptions(o interface{}) error {
 		v, err := util.ParseKubernetesVersion(clusterSpec.KubernetesVersion)
 		if err == nil {
 			switch v.Minor {
-			case 23:
-				image = "registry.k8s.io/autoscaling/cluster-autoscaler:v1.23.1"
 			case 24:
 				image = "registry.k8s.io/autoscaling/cluster-autoscaler:v1.24.3"
 			case 25:
 				image = "registry.k8s.io/autoscaling/cluster-autoscaler:v1.25.3"
 			case 26:
-				image = "registry.k8s.io/autoscaling/cluster-autoscaler:v1.26.4"
+				image = "registry.k8s.io/autoscaling/cluster-autoscaler:v1.26.8"
 			case 27:
-				image = "registry.k8s.io/autoscaling/cluster-autoscaler:v1.27.3"
+				image = "registry.k8s.io/autoscaling/cluster-autoscaler:v1.27.7"
 			case 28:
-				image = "registry.k8s.io/autoscaling/cluster-autoscaler:v1.28.0"
+				image = "registry.k8s.io/autoscaling/cluster-autoscaler:v1.28.4"
+			case 29:
+				image = "registry.k8s.io/autoscaling/cluster-autoscaler:v1.29.2"
+			case 30:
+				image = "registry.k8s.io/autoscaling/cluster-autoscaler:v1.30.0"
 			default:
-				image = "registry.k8s.io/autoscaling/cluster-autoscaler:v1.28.0"
+				image = "registry.k8s.io/autoscaling/cluster-autoscaler:v1.30.0"
 			}
 		}
 		cas.Image = fi.PtrTo(image)
@@ -71,6 +73,9 @@ func (b *ClusterAutoscalerOptionsBuilder) BuildOptions(o interface{}) error {
 	if cas.ScaleDownUtilizationThreshold == nil {
 		cas.ScaleDownUtilizationThreshold = fi.PtrTo("0.5")
 	}
+	if cas.SkipNodesWithCustomControllerPods == nil {
+		cas.SkipNodesWithCustomControllerPods = fi.PtrTo(true)
+	}
 	if cas.SkipNodesWithLocalStorage == nil {
 		cas.SkipNodesWithLocalStorage = fi.PtrTo(true)
 	}
