@@ -17,17 +17,20 @@ limitations under the License.
 package awstasks
 
 import (
+	"context"
 	"fmt"
 	"sort"
 	"strconv"
 	"strings"
+	"time"
 
 	"github.com/aws/aws-sdk-go/aws"
-	"github.com/aws/aws-sdk-go/aws/awserr"
 	"github.com/aws/aws-sdk-go/service/elb"
 	"github.com/aws/aws-sdk-go/service/elbv2"
 	"github.com/aws/aws-sdk-go/service/route53"
 	"k8s.io/klog/v2"
+	"k8s.io/kops/pkg/truncate"
+	"k8s.io/kops/pkg/wellknownservices"
 	"k8s.io/kops/upup/pkg/fi"
 	"k8s.io/kops/upup/pkg/fi/cloudup/awsup"
 	"k8s.io/kops/upup/pkg/fi/cloudup/terraform"
@@ -44,18 +47,20 @@ type NetworkLoadBalancer struct {
 	Name      *string
 	Lifecycle fi.Lifecycle
 
-	// LoadBalancerName is the name in NLB, possibly different from our name
+	// LoadBalancerBaseName is the base name to use when naming load balancers in NLB.
+	// The full, stable name will be in the Name tag.
 	// (NLB is restricted as to names, so we have limited choices!)
-	// We use the Name tag to find the existing NLB.
-	LoadBalancerName *string
-	CLBName          *string
+	LoadBalancerBaseName *string
+
+	// CLBName is the name of a ClassicLoadBalancer to delete, if found.
+	// This enables migration from CLB -> NLB
+	CLBName *string
 
 	DNSName      *string
 	HostedZoneId *string
 
 	SubnetMappings []*SubnetMapping
-
-	Listeners []*NetworkLoadBalancerListener
+	SecurityGroups []*SecurityGroup
 
 	Scheme *string
 
@@ -63,14 +68,32 @@ type NetworkLoadBalancer struct {
 
 	IpAddressType *string
 
-	Tags         map[string]string
-	ForAPIServer bool
+	Tags map[string]string
 
 	Type *string
 
-	VPC          *VPC
-	TargetGroups []*TargetGroup
-	AccessLog    *NetworkLoadBalancerAccessLog
+	VPC       *VPC
+	AccessLog *NetworkLoadBalancerAccessLog
+
+	// WellKnownServices indicates which services are supported by this resource.
+	// This field is internal and is not rendered to the cloud.
+	WellKnownServices []wellknownservices.WellKnownService
+
+	// waitForLoadBalancerReady controls whether we wait for the load balancer to be ready before completing the "Render" operation.
+	waitForLoadBalancerReady bool
+
+	// After this is found/created, we store the ARN
+	loadBalancerArn string
+
+	// After this is found/created, we store the revision
+	revision string
+
+	// deletions is a list of previous versions of this object, that we should delete when asked to clean up.
+	deletions []fi.CloudupDeletion
+}
+
+func (e *NetworkLoadBalancer) SetWaitForLoadBalancerReady(v bool) {
+	e.waitForLoadBalancerReady = v
 }
 
 var _ fi.CompareWithID = &NetworkLoadBalancer{}
@@ -81,103 +104,9 @@ func (e *NetworkLoadBalancer) CompareWithID() *string {
 	return e.Name
 }
 
-type NetworkLoadBalancerListener struct {
-	Port             int
-	TargetGroupName  string
-	SSLCertificateID string
-	SSLPolicy        string
-}
-
-func (e *NetworkLoadBalancerListener) mapToAWS(targetGroups []*TargetGroup, loadBalancerArn string) (*elbv2.CreateListenerInput, error) {
-	var tgARN string
-	for _, tg := range targetGroups {
-		if fi.ValueOf(tg.Name) == e.TargetGroupName {
-			tgARN = fi.ValueOf(tg.ARN)
-		}
-	}
-	if tgARN == "" {
-		return nil, fmt.Errorf("target group not found for NLB listener %+v", e)
-	}
-
-	l := &elbv2.CreateListenerInput{
-		DefaultActions: []*elbv2.Action{
-			{
-				TargetGroupArn: aws.String(tgARN),
-				Type:           aws.String(elbv2.ActionTypeEnumForward),
-			},
-		},
-		LoadBalancerArn: aws.String(loadBalancerArn),
-		Port:            aws.Int64(int64(e.Port)),
-	}
-
-	if e.SSLCertificateID != "" {
-		l.Certificates = []*elbv2.Certificate{}
-		l.Certificates = append(l.Certificates, &elbv2.Certificate{
-			CertificateArn: aws.String(e.SSLCertificateID),
-		})
-		l.Protocol = aws.String(elbv2.ProtocolEnumTls)
-		if e.SSLPolicy != "" {
-			l.SslPolicy = aws.String(e.SSLPolicy)
-		}
-	} else {
-		l.Protocol = aws.String(elbv2.ProtocolEnumTcp)
-	}
-
-	return l, nil
-}
-
-var _ fi.CloudupHasDependencies = &NetworkLoadBalancerListener{}
-
-func (e *NetworkLoadBalancerListener) GetDependencies(tasks map[string]fi.CloudupTask) []fi.CloudupTask {
-	return nil
-}
-
-// OrderListenersByPort implements sort.Interface for []OrderListenersByPort, based on port number
-type OrderListenersByPort []*NetworkLoadBalancerListener
-
-func (a OrderListenersByPort) Len() int      { return len(a) }
-func (a OrderListenersByPort) Swap(i, j int) { a[i], a[j] = a[j], a[i] }
-func (a OrderListenersByPort) Less(i, j int) bool {
-	return a[i].Port < a[j].Port
-}
-
-// The load balancer name 'api.renamenlbcluster.k8s.local' can only contain characters that are alphanumeric characters and hyphens(-)\n\tstatus code: 400,
-func findNetworkLoadBalancerByLoadBalancerName(cloud awsup.AWSCloud, loadBalancerName string) (*elbv2.LoadBalancer, error) {
-	request := &elbv2.DescribeLoadBalancersInput{
-		Names: []*string{&loadBalancerName},
-	}
-	found, err := describeNetworkLoadBalancers(cloud, request, func(lb *elbv2.LoadBalancer) bool {
-		// TODO: Filter by cluster?
-
-		if aws.StringValue(lb.LoadBalancerName) == loadBalancerName {
-			return true
-		}
-
-		klog.Warningf("Got NLB with unexpected name: %q", aws.StringValue(lb.LoadBalancerName))
-		return false
-	})
-	if err != nil {
-		if awsError, ok := err.(awserr.Error); ok {
-			if awsError.Code() == "LoadBalancerNotFound" {
-				return nil, nil
-			}
-		}
-
-		return nil, fmt.Errorf("error listing NLBs: %v", err)
-	}
-
-	if len(found) == 0 {
-		return nil, nil
-	}
-
-	if len(found) != 1 {
-		return nil, fmt.Errorf("Found multiple NLBs with name %q", loadBalancerName)
-	}
-
-	return found[0], nil
-}
-
 func findNetworkLoadBalancerByAlias(cloud awsup.AWSCloud, alias *route53.AliasTarget) (*elbv2.LoadBalancer, error) {
+	ctx := context.TODO()
+
 	// TODO: Any way to avoid listing all NLBs?
 	request := &elbv2.DescribeLoadBalancersInput{}
 
@@ -189,9 +118,7 @@ func findNetworkLoadBalancerByAlias(cloud awsup.AWSCloud, alias *route53.AliasTa
 
 	matchHostedZoneId := aws.StringValue(alias.HostedZoneId)
 
-	found, err := describeNetworkLoadBalancers(cloud, request, func(lb *elbv2.LoadBalancer) bool {
-		// TODO: Filter by cluster?
-
+	found, err := describeNetworkLoadBalancers(ctx, cloud, request, func(lb *elbv2.LoadBalancer) bool {
 		if matchHostedZoneId != aws.StringValue(lb.CanonicalHostedZoneId) {
 			return false
 		}
@@ -215,9 +142,9 @@ func findNetworkLoadBalancerByAlias(cloud awsup.AWSCloud, alias *route53.AliasTa
 	return found[0], nil
 }
 
-func describeNetworkLoadBalancers(cloud awsup.AWSCloud, request *elbv2.DescribeLoadBalancersInput, filter func(*elbv2.LoadBalancer) bool) ([]*elbv2.LoadBalancer, error) {
+func describeNetworkLoadBalancers(ctx context.Context, cloud awsup.AWSCloud, request *elbv2.DescribeLoadBalancersInput, filter func(*elbv2.LoadBalancer) bool) ([]*elbv2.LoadBalancer, error) {
 	var found []*elbv2.LoadBalancer
-	err := cloud.ELBV2().DescribeLoadBalancersPages(request, func(p *elbv2.DescribeLoadBalancersOutput, lastPage bool) (shouldContinue bool) {
+	err := cloud.ELBV2().DescribeLoadBalancersPagesWithContext(ctx, request, func(p *elbv2.DescribeLoadBalancersOutput, lastPage bool) (shouldContinue bool) {
 		for _, lb := range p.LoadBalancers {
 			if filter(lb) {
 				found = append(found, lb)
@@ -242,22 +169,44 @@ func (e *NetworkLoadBalancer) getHostedZoneId() *string {
 }
 
 func (e *NetworkLoadBalancer) Find(c *fi.CloudupContext) (*NetworkLoadBalancer, error) {
+	ctx := c.Context()
 	cloud := c.T.Cloud.(awsup.AWSCloud)
 
-	lb, err := cloud.FindELBV2ByNameTag(e.Tags["Name"])
+	allLoadBalancers, err := awsup.ListELBV2LoadBalancers(ctx, cloud)
 	if err != nil {
 		return nil, err
 	}
-	if lb == nil {
+
+	latest := awsup.FindLatestELBV2ByNameTag(allLoadBalancers, fi.ValueOf(e.Name))
+	if err != nil {
+		return nil, err
+	}
+
+	// Stash deletions for later
+	for _, lb := range allLoadBalancers {
+		if lb.NameTag() != fi.ValueOf(e.Name) {
+			continue
+		}
+		if latest != nil && latest.ARN() == lb.ARN() {
+			continue
+		}
+
+		e.deletions = append(e.deletions, &deleteNLB{
+			obj: lb,
+		})
+	}
+
+	if latest == nil {
 		return nil, nil
 	}
 
-	loadBalancerArn := lb.LoadBalancerArn
+	lb := latest.LoadBalancer
+
+	loadBalancerArn := latest.ARN()
 
 	actual := &NetworkLoadBalancer{}
 	actual.Name = e.Name
 	actual.CLBName = e.CLBName
-	actual.LoadBalancerName = lb.LoadBalancerName
 	actual.DNSName = lb.DNSName
 	actual.HostedZoneId = lb.CanonicalHostedZoneId // CanonicalHostedZoneNameID
 	actual.Scheme = lb.Scheme
@@ -265,16 +214,16 @@ func (e *NetworkLoadBalancer) Find(c *fi.CloudupContext) (*NetworkLoadBalancer,
 	actual.Type = lb.Type
 	actual.IpAddressType = lb.IpAddressType
 
-	tagMap, err := cloud.DescribeELBV2Tags([]string{*loadBalancerArn})
-	if err != nil {
-		return nil, err
-	}
 	actual.Tags = make(map[string]string)
-	for _, tag := range tagMap[*loadBalancerArn] {
-		if strings.HasPrefix(aws.StringValue(tag.Key), "aws:cloudformation:") {
+	for _, tag := range latest.Tags {
+		k := aws.StringValue(tag.Key)
+		if strings.HasPrefix(k, "aws:cloudformation:") {
 			continue
 		}
-		actual.Tags[aws.StringValue(tag.Key)] = aws.StringValue(tag.Value)
+		if k == awsup.KopsResourceRevisionTag {
+			continue
+		}
+		actual.Tags[k] = aws.StringValue(tag.Value)
 	}
 
 	for _, az := range lb.AvailabilityZones {
@@ -298,59 +247,12 @@ func (e *NetworkLoadBalancer) Find(c *fi.CloudupContext) (*NetworkLoadBalancer,
 		actual.SubnetMappings = append(actual.SubnetMappings, sm)
 	}
 
-	{
-		request := &elbv2.DescribeListenersInput{
-			LoadBalancerArn: loadBalancerArn,
-		}
-		response, err := cloud.ELBV2().DescribeListeners(request)
-		if err != nil {
-			return nil, fmt.Errorf("error querying for NLB listeners :%v", err)
-		}
-
-		actual.Listeners = []*NetworkLoadBalancerListener{}
-		actual.TargetGroups = []*TargetGroup{}
-		for _, l := range response.Listeners {
-			actualListener := &NetworkLoadBalancerListener{}
-			actualListener.Port = int(aws.Int64Value(l.Port))
-			if len(l.Certificates) != 0 {
-				actualListener.SSLCertificateID = aws.StringValue(l.Certificates[0].CertificateArn) // What if there is more then one certificate, can we just grab the default certificate? we don't set it as default, we only set the one.
-				if l.SslPolicy != nil {
-					actualListener.SSLPolicy = aws.StringValue(l.SslPolicy)
-				}
-			}
-
-			// This will need to be rearranged when we recognized multiple listeners and target groups per NLB
-			if len(l.DefaultActions) > 0 {
-				targetGroupARN := l.DefaultActions[0].TargetGroupArn
-				if targetGroupARN != nil {
-					targetGroupName, err := awsup.GetTargetGroupNameFromARN(fi.ValueOf(targetGroupARN))
-					if err != nil {
-						return nil, err
-					}
-					actual.TargetGroups = append(actual.TargetGroups, &TargetGroup{ARN: targetGroupARN, Name: fi.PtrTo(targetGroupName)})
-
-					cloud := c.T.Cloud.(awsup.AWSCloud)
-					descResp, err := cloud.ELBV2().DescribeTargetGroups(&elbv2.DescribeTargetGroupsInput{
-						TargetGroupArns: []*string{targetGroupARN},
-					})
-					if err != nil {
-						return nil, fmt.Errorf("error querying for NLB listener target groups: %v", err)
-					}
-					if len(descResp.TargetGroups) != 1 {
-						return nil, fmt.Errorf("unexpected DescribeTargetGroups response: %v", descResp)
-					}
-
-					actualListener.TargetGroupName = aws.StringValue(descResp.TargetGroups[0].TargetGroupName)
-				}
-			}
-			actual.Listeners = append(actual.Listeners, actualListener)
-		}
-		sort.Stable(OrderTargetGroupsByName(actual.TargetGroups))
-
+	for _, sg := range lb.SecurityGroups {
+		actual.SecurityGroups = append(actual.SecurityGroups, &SecurityGroup{ID: sg})
 	}
 
 	{
-		lbAttributes, err := findNetworkLoadBalancerAttributes(cloud, aws.StringValue(loadBalancerArn))
+		lbAttributes, err := findNetworkLoadBalancerAttributes(cloud, loadBalancerArn)
 		if err != nil {
 			return nil, err
 		}
@@ -406,61 +308,77 @@ func (e *NetworkLoadBalancer) Find(c *fi.CloudupContext) (*NetworkLoadBalancer,
 	if e.HostedZoneId == nil {
 		e.HostedZoneId = actual.HostedZoneId
 	}
-	if e.LoadBalancerName == nil {
-		e.LoadBalancerName = actual.LoadBalancerName
-	}
 
 	// An existing internal NLB can't be updated to dualstack.
 	if fi.ValueOf(actual.Scheme) == elbv2.LoadBalancerSchemeEnumInternal && fi.ValueOf(actual.IpAddressType) == elbv2.IpAddressTypeIpv4 {
 		e.IpAddressType = actual.IpAddressType
 	}
 
-	// We allow for the LoadBalancerName to be wrong:
-	// 1. We don't want to force a rename of the NLB, because that is a destructive operation
-	if fi.ValueOf(e.LoadBalancerName) != fi.ValueOf(actual.LoadBalancerName) {
-		klog.V(2).Infof("Reusing existing load balancer with name: %q", aws.StringValue(actual.LoadBalancerName))
-		e.LoadBalancerName = actual.LoadBalancerName
-	}
-
 	_ = actual.Normalize(c)
-	actual.ForAPIServer = e.ForAPIServer
+	actual.WellKnownServices = e.WellKnownServices
 	actual.Lifecycle = e.Lifecycle
+	actual.LoadBalancerBaseName = e.LoadBalancerBaseName
+
+	// Store state for other tasks
+	e.loadBalancerArn = aws.StringValue(lb.LoadBalancerArn)
+	actual.loadBalancerArn = e.loadBalancerArn
+	e.revision, _ = latest.GetTag(awsup.KopsResourceRevisionTag)
+	actual.revision = e.revision
 
 	klog.V(4).Infof("Found NLB %+v", actual)
 
+	// AWS does not allow us to add security groups to an ELB that was initially created without them.
+	// This forces a new revision (currently, the only operation that forces a new revision)
+	if len(actual.SecurityGroups) == 0 && len(e.SecurityGroups) > 0 {
+		klog.Warningf("setting securityGroups on an existing NLB created without securityGroups; will force a new NLB")
+		t := time.Now()
+		revision := strconv.FormatInt(t.Unix(), 10)
+		actual = nil
+		e.revision = revision
+	}
+
 	return actual, nil
 }
 
 var _ fi.HasAddress = &NetworkLoadBalancer{}
 
-func (e *NetworkLoadBalancer) IsForAPIServer() bool {
-	return e.ForAPIServer
+// GetWellKnownServices implements fi.HasAddress::GetWellKnownServices.
+// It indicates which services we support with this load balancer.
+func (e *NetworkLoadBalancer) GetWellKnownServices() []wellknownservices.WellKnownService {
+	return e.WellKnownServices
 }
 
-func (e *NetworkLoadBalancer) FindAddresses(context *fi.CloudupContext) ([]string, error) {
+func (e *NetworkLoadBalancer) FindAddresses(c *fi.CloudupContext) ([]string, error) {
+	ctx := c.Context()
+
 	var addresses []string
 
-	cloud := context.T.Cloud.(awsup.AWSCloud)
-	cluster := context.T.Cluster
+	cloud := c.T.Cloud.(awsup.AWSCloud)
+	cluster := c.T.Cluster
 
 	{
-		lb, err := cloud.FindELBV2ByNameTag(e.Tags["Name"])
+		allLoadBalancers, err := awsup.ListELBV2LoadBalancers(ctx, cloud)
 		if err != nil {
-			return nil, fmt.Errorf("failed to find load balancer matching %q: %w", e.Tags["Name"], err)
+			return nil, err
 		}
-		if lb != nil && fi.ValueOf(lb.DNSName) != "" {
-			addresses = append(addresses, fi.ValueOf(lb.DNSName))
-		}
-	}
 
-	if cluster.UsesNoneDNS() {
-		nis, err := cloud.FindELBV2NetworkInterfacesByName(fi.ValueOf(e.VPC.ID), fi.ValueOf(e.LoadBalancerName))
-		if err != nil {
-			return nil, fmt.Errorf("failed to find network interfaces matching %q: %w", fi.ValueOf(e.LoadBalancerName), err)
-		}
-		for _, ni := range nis {
-			if fi.ValueOf(ni.PrivateIpAddress) != "" {
-				addresses = append(addresses, fi.ValueOf(ni.PrivateIpAddress))
+		lb := awsup.FindLatestELBV2ByNameTag(allLoadBalancers, fi.ValueOf(e.Name))
+
+		if lb != nil {
+			if fi.ValueOf(lb.LoadBalancer.DNSName) != "" {
+				addresses = append(addresses, fi.ValueOf(lb.LoadBalancer.DNSName))
+			}
+
+			if cluster.UsesNoneDNS() {
+				nis, err := cloud.FindELBV2NetworkInterfacesByName(fi.ValueOf(e.VPC.ID), aws.StringValue(lb.LoadBalancer.LoadBalancerName))
+				if err != nil {
+					return nil, fmt.Errorf("failed to find network interfaces matching %q: %w", aws.StringValue(lb.LoadBalancer.LoadBalancerName), err)
+				}
+				for _, ni := range nis {
+					if fi.ValueOf(ni.PrivateIpAddress) != "" {
+						addresses = append(addresses, fi.ValueOf(ni.PrivateIpAddress))
+					}
+				}
 			}
 		}
 	}
@@ -477,8 +395,6 @@ func (e *NetworkLoadBalancer) Run(c *fi.CloudupContext) error {
 func (e *NetworkLoadBalancer) Normalize(c *fi.CloudupContext) error {
 	// We need to sort our arrays consistently, so we don't get spurious changes
 	sort.Stable(OrderSubnetMappingsByName(e.SubnetMappings))
-	sort.Stable(OrderListenersByPort(e.Listeners))
-	sort.Stable(OrderTargetGroupsByName(e.TargetGroups))
 
 	e.IpAddressType = fi.PtrTo("dualstack")
 	for _, subnet := range e.SubnetMappings {
@@ -545,32 +461,44 @@ func (*NetworkLoadBalancer) CheckChanges(a, e, changes *NetworkLoadBalancer) err
 }
 
 func (_ *NetworkLoadBalancer) RenderAWS(t *awsup.AWSAPITarget, a, e, changes *NetworkLoadBalancer) error {
-	var loadBalancerName string
-	var loadBalancerArn string
+	ctx := context.TODO()
 
-	if len(e.Listeners) != len(e.TargetGroups) {
-		return fmt.Errorf("nlb listeners and target groups do not match: %v listeners vs %v target groups", len(e.Listeners), len(e.TargetGroups))
+	loadBalancerArn := ""
+
+	revision := e.revision
+
+	// TODO: Use maps.Clone when we are >= go1.21 on supported branches
+	tags := make(map[string]string)
+	for k, v := range e.Tags {
+		tags[k] = v
+	}
+
+	// We removed revision for the diff/plan, but we want to set it
+	if revision != "" {
+		tags[awsup.KopsResourceRevisionTag] = revision
 	}
 
 	if a == nil {
-		if e.LoadBalancerName == nil {
-			return fi.RequiredField("LoadBalancerName")
-		}
-		for _, tg := range e.TargetGroups {
-			if tg.ARN == nil {
-				return fmt.Errorf("missing required target group ARN for NLB creation %v", tg)
-			}
-		}
+		loadBalancerName := fi.ValueOf(e.LoadBalancerBaseName)
+		if revision != "" {
+			s := fi.ValueOf(e.LoadBalancerBaseName) + "-" + revision
 
-		loadBalancerName = *e.LoadBalancerName
+			// We always compute the hash and add it, lest we trick users into assuming that we never do this
+			opt := truncate.TruncateStringOptions{
+				MaxLength:     32,
+				AlwaysAddHash: true,
+				HashLength:    6,
+			}
+			loadBalancerName = truncate.TruncateString(s, opt)
+		}
 
 		{
 			request := &elbv2.CreateLoadBalancerInput{}
-			request.Name = e.LoadBalancerName
+			request.Name = &loadBalancerName
 			request.Scheme = e.Scheme
 			request.Type = e.Type
 			request.IpAddressType = e.IpAddressType
-			request.Tags = awsup.ELBv2Tags(e.Tags)
+			request.Tags = awsup.ELBv2Tags(tags)
 
 			for _, subnetMapping := range e.SubnetMappings {
 				request.SubnetMappings = append(request.SubnetMappings, &elbv2.SubnetMapping{
@@ -580,6 +508,10 @@ func (_ *NetworkLoadBalancer) RenderAWS(t *awsup.AWSAPITarget, a, e, changes *Ne
 				})
 			}
 
+			for _, sg := range e.SecurityGroups {
+				request.SecurityGroups = append(request.SecurityGroups, sg.ID)
+			}
+
 			klog.V(2).Infof("Creating NLB %q", loadBalancerName)
 
 			response, err := t.Cloud.ELBV2().CreateLoadBalancer(request)
@@ -594,54 +526,30 @@ func (_ *NetworkLoadBalancer) RenderAWS(t *awsup.AWSAPITarget, a, e, changes *Ne
 			e.DNSName = lb.DNSName
 			e.HostedZoneId = lb.CanonicalHostedZoneId
 			e.VPC = &VPC{ID: lb.VpcId}
-			loadBalancerArn = fi.ValueOf(lb.LoadBalancerArn)
-
+			loadBalancerArn = aws.StringValue(lb.LoadBalancerArn)
+			e.loadBalancerArn = loadBalancerArn
+			e.revision = revision
 		}
 
-		// Wait for all load balancer components to be created (including network interfaces needed for NoneDNS).
-		// Limiting this to clusters using NoneDNS because load balancer creation is quite slow.
-		for _, tg := range e.TargetGroups {
-			if strings.HasPrefix(fi.ValueOf(tg.Name), "kops-controller") {
-				klog.Infof("Waiting for load balancer %q to be created...", loadBalancerName)
-				request := &elbv2.DescribeLoadBalancersInput{
-					Names: []*string{&loadBalancerName},
-				}
-				err := t.Cloud.ELBV2().WaitUntilLoadBalancerAvailable(request)
-				if err != nil {
-					return fmt.Errorf("error waiting for NLB %q: %w", loadBalancerName, err)
-				}
-				break
+		if e.waitForLoadBalancerReady {
+			klog.Infof("Waiting for load balancer %q to be created...", loadBalancerName)
+			request := &elbv2.DescribeLoadBalancersInput{
+				Names: []*string{&loadBalancerName},
+			}
+
+			err := t.Cloud.ELBV2().WaitUntilLoadBalancerAvailableWithContext(ctx, request)
+			if err != nil {
+				return fmt.Errorf("error waiting for NLB %q: %w", loadBalancerName, err)
 			}
 		}
 
-		{
-			for _, listener := range e.Listeners {
-				createListenerInput, err := listener.mapToAWS(e.TargetGroups, loadBalancerArn)
-				if err != nil {
-					return err
-				}
-
-				klog.V(2).Infof("Creating Listener for NLB with port %v", listener.Port)
-				_, err = t.Cloud.ELBV2().CreateListener(createListenerInput)
-				if err != nil {
-					return fmt.Errorf("error creating listener for NLB: %v", err)
-				}
-			}
-		}
 	} else {
-		loadBalancerName = fi.ValueOf(a.LoadBalancerName)
-
-		lb, err := findNetworkLoadBalancerByLoadBalancerName(t.Cloud, loadBalancerName)
-		if err != nil {
-			return fmt.Errorf("error getting load balancer by name: %v", err)
-		}
-
-		loadBalancerArn = fi.ValueOf(lb.LoadBalancerArn)
+		loadBalancerArn = a.loadBalancerArn
 
 		if changes.IpAddressType != nil {
 			request := &elbv2.SetIpAddressTypeInput{
 				IpAddressType:   e.IpAddressType,
-				LoadBalancerArn: lb.LoadBalancerArn,
+				LoadBalancerArn: &loadBalancerArn,
 			}
 			if _, err := t.Cloud.ELBV2().SetIpAddressType(request); err != nil {
 				return fmt.Errorf("error setting the IP addresses type: %v", err)
@@ -686,48 +594,25 @@ func (_ *NetworkLoadBalancer) RenderAWS(t *awsup.AWSAPITarget, a, e, changes *Ne
 			}
 		}
 
-		if changes.Listeners != nil {
-
-			if lb != nil {
-
-				request := &elbv2.DescribeListenersInput{
-					LoadBalancerArn: lb.LoadBalancerArn,
-				}
-				response, err := t.Cloud.ELBV2().DescribeListeners(request)
-				if err != nil {
-					return fmt.Errorf("error querying for NLB listeners :%v", err)
-				}
-
-				for _, l := range response.Listeners {
-					// delete the listener before recreating it
-					_, err := t.Cloud.ELBV2().DeleteListener(&elbv2.DeleteListenerInput{
-						ListenerArn: l.ListenerArn,
-					})
-					if err != nil {
-						return fmt.Errorf("error deleting load balancer listener with arn = : %v : %v", l.ListenerArn, err)
-					}
-				}
+		if changes.SecurityGroups != nil {
+			request := &elbv2.SetSecurityGroupsInput{
+				LoadBalancerArn: &loadBalancerArn,
+			}
+			for _, sg := range e.SecurityGroups {
+				request.SecurityGroups = append(request.SecurityGroups, sg.ID)
 			}
 
-			for _, listener := range changes.Listeners {
-
-				awsListener, err := listener.mapToAWS(e.TargetGroups, loadBalancerArn)
-				if err != nil {
-					return err
-				}
-
-				klog.V(2).Infof("Creating Listener for NLB with port %v", listener.Port)
-				_, err = t.Cloud.ELBV2().CreateListener(awsListener)
-				if err != nil {
-					return fmt.Errorf("error creating NLB listener: %v", err)
-				}
+			klog.V(2).Infof("Updating Load Balancer Security Groups")
+			if _, err := t.Cloud.ELBV2().SetSecurityGroups(request); err != nil {
+				return fmt.Errorf("Error updating security groups on Load Balancer: %v", err)
 			}
 		}
-		if err := t.AddELBV2Tags(loadBalancerArn, e.Tags); err != nil {
+
+		if err := t.AddELBV2Tags(loadBalancerArn, tags); err != nil {
 			return err
 		}
 
-		if err := t.RemoveELBV2Tags(loadBalancerArn, e.Tags); err != nil {
+		if err := t.RemoveELBV2Tags(loadBalancerArn, tags); err != nil {
 			return err
 		}
 	}
@@ -744,6 +629,7 @@ type terraformNetworkLoadBalancer struct {
 	Internal               bool                                        `cty:"internal"`
 	Type                   string                                      `cty:"load_balancer_type"`
 	IPAddressType          *string                                     `cty:"ip_address_type"`
+	SecurityGroups         []*terraformWriter.Literal                  `cty:"security_groups"`
 	SubnetMappings         []terraformNetworkLoadBalancerSubnetMapping `cty:"subnet_mapping"`
 	CrossZoneLoadBalancing bool                                        `cty:"enable_cross_zone_load_balancing"`
 	AccessLog              *terraformNetworkLoadBalancerAccessLog      `cty:"access_logs"`
@@ -757,23 +643,9 @@ type terraformNetworkLoadBalancerSubnetMapping struct {
 	PrivateIPv4Address *string                  `cty:"private_ipv4_address"`
 }
 
-type terraformNetworkLoadBalancerListener struct {
-	LoadBalancer   *terraformWriter.Literal                     `cty:"load_balancer_arn"`
-	Port           int64                                        `cty:"port"`
-	Protocol       string                                       `cty:"protocol"`
-	CertificateARN *string                                      `cty:"certificate_arn"`
-	SSLPolicy      *string                                      `cty:"ssl_policy"`
-	DefaultAction  []terraformNetworkLoadBalancerListenerAction `cty:"default_action"`
-}
-
-type terraformNetworkLoadBalancerListenerAction struct {
-	Type           string                   `cty:"type"`
-	TargetGroupARN *terraformWriter.Literal `cty:"target_group_arn"`
-}
-
 func (_ *NetworkLoadBalancer) RenderTerraform(t *terraform.TerraformTarget, a, e, changes *NetworkLoadBalancer) error {
 	nlbTF := &terraformNetworkLoadBalancer{
-		Name:                   *e.LoadBalancerName,
+		Name:                   *e.LoadBalancerBaseName,
 		Internal:               fi.ValueOf(e.Scheme) == elbv2.LoadBalancerSchemeEnumInternal,
 		Type:                   elbv2.LoadBalancerTypeEnumNetwork,
 		Tags:                   e.Tags,
@@ -791,6 +663,11 @@ func (_ *NetworkLoadBalancer) RenderTerraform(t *terraform.TerraformTarget, a, e
 		})
 	}
 
+	for _, sg := range e.SecurityGroups {
+		nlbTF.SecurityGroups = append(nlbTF.SecurityGroups, sg.TerraformLink())
+	}
+	terraformWriter.SortLiterals(nlbTF.SecurityGroups)
+
 	if e.AccessLog != nil && fi.ValueOf(e.AccessLog.Enabled) {
 		nlbTF.AccessLog = &terraformNetworkLoadBalancerAccessLog{
 			Enabled:        e.AccessLog.Enabled,
@@ -799,88 +676,48 @@ func (_ *NetworkLoadBalancer) RenderTerraform(t *terraform.TerraformTarget, a, e
 		}
 	}
 
-	err := t.RenderResource("aws_lb", *e.Name, nlbTF)
+	err := t.RenderResource("aws_lb", e.TerraformName(), nlbTF)
 	if err != nil {
 		return err
 	}
 
-	for _, listener := range e.Listeners {
-		var listenerTG *TargetGroup
-		for _, tg := range e.TargetGroups {
-			if aws.StringValue(tg.Name) == listener.TargetGroupName {
-				listenerTG = tg
-				break
-			}
-		}
-		if listenerTG == nil {
-			return fmt.Errorf("target group not found for NLB listener %+v", e)
-		}
-		listenerTF := &terraformNetworkLoadBalancerListener{
-			LoadBalancer: e.TerraformLink(),
-			Port:         int64(listener.Port),
-			DefaultAction: []terraformNetworkLoadBalancerListenerAction{
-				{
-					Type:           elbv2.ActionTypeEnumForward,
-					TargetGroupARN: listenerTG.TerraformLink(),
-				},
-			},
-		}
-		if listener.SSLCertificateID != "" {
-			listenerTF.CertificateARN = &listener.SSLCertificateID
-			listenerTF.Protocol = elbv2.ProtocolEnumTls
-			if listener.SSLPolicy != "" {
-				listenerTF.SSLPolicy = &listener.SSLPolicy
-			}
-		} else {
-			listenerTF.Protocol = elbv2.ProtocolEnumTcp
-		}
-
-		err = t.RenderResource("aws_lb_listener", fmt.Sprintf("%v-%v", *e.Name, listener.Port), listenerTF)
-		if err != nil {
-			return err
-		}
-	}
-
 	return nil
 }
 
+func (e *NetworkLoadBalancer) TerraformName() string {
+	tfName := strings.Replace(fi.ValueOf(e.Name), ".", "-", -1)
+	return tfName
+}
+
 func (e *NetworkLoadBalancer) TerraformLink(params ...string) *terraformWriter.Literal {
 	prop := "id"
 	if len(params) > 0 {
 		prop = params[0]
 	}
-	return terraformWriter.LiteralProperty("aws_lb", *e.Name, prop)
+	return terraformWriter.LiteralProperty("aws_lb", e.TerraformName(), prop)
 }
 
 // FindDeletions schedules deletion of the corresponding legacy classic load balancer when it no longer has targets.
 func (e *NetworkLoadBalancer) FindDeletions(context *fi.CloudupContext) ([]fi.CloudupDeletion, error) {
-	if e.CLBName == nil {
-		return nil, nil
+	var deletions []fi.CloudupDeletion
+
+	deletions = append(deletions, e.deletions...)
+
+	if e.CLBName != nil {
+		cloud := context.T.Cloud.(awsup.AWSCloud)
+
+		lb, err := cloud.FindELBByNameTag(fi.ValueOf(e.CLBName))
+		if err != nil {
+			return nil, err
+		}
+
+		if lb != nil {
+			klog.V(4).Infof("Found CLB %v", aws.StringValue(lb.LoadBalancerName))
+			deletions = append(deletions, &deleteClassicLoadBalancer{LoadBalancerName: e.CLBName})
+		}
 	}
 
-	cloud := context.T.Cloud.(awsup.AWSCloud)
-
-	lb, err := cloud.FindELBByNameTag(fi.ValueOf(e.CLBName))
-	if err != nil {
-		return nil, err
-	}
-	if lb == nil {
-		return nil, nil
-	}
-
-	// Testing shows that the instances are deregistered immediately after the apply_cluster.
-	// TODO: Figure out how to delay deregistration until instances are terminated.
-	//if len(lb.Instances) > 0 {
-	//	klog.V(2).Infof("CLB %s has targets; not scheduling deletion", *lb.LoadBalancerName)
-	//	return nil, nil
-	//}
-
-	actual := &deleteClassicLoadBalancer{}
-	actual.LoadBalancerName = lb.LoadBalancerName
-
-	klog.V(4).Infof("Found CLB %+v", actual)
-
-	return []fi.CloudupDeletion{actual}, nil
+	return deletions, nil
 }
 
 type deleteClassicLoadBalancer struct {
@@ -889,6 +726,18 @@ type deleteClassicLoadBalancer struct {
 	LoadBalancerName *string
 }
 
+func (d deleteClassicLoadBalancer) TaskName() string {
+	return "ClassicLoadBalancer"
+}
+
+func (d deleteClassicLoadBalancer) Item() string {
+	return *d.LoadBalancerName
+}
+
+func (d deleteClassicLoadBalancer) DeferDeletion() bool {
+	return true
+}
+
 func (d deleteClassicLoadBalancer) Delete(t fi.CloudupTarget) error {
 	awsTarget, ok := t.(*awsup.AWSAPITarget)
 	if !ok {
@@ -905,10 +754,54 @@ func (d deleteClassicLoadBalancer) Delete(t fi.CloudupTarget) error {
 	return nil
 }
 
-func (d deleteClassicLoadBalancer) TaskName() string {
-	return "ClassicLoadBalancer"
+// deleteNLB tracks a NLB that we're going to delete
+// It implements fi.CloudupDeletion
+type deleteNLB struct {
+	obj *awsup.LoadBalancerInfo
 }
 
-func (d deleteClassicLoadBalancer) Item() string {
-	return *d.LoadBalancerName
+func buildDeleteNLB(obj *awsup.LoadBalancerInfo) *deleteNLB {
+	d := &deleteNLB{}
+	d.obj = obj
+	return d
+}
+
+var _ fi.CloudupDeletion = &deleteNLB{}
+
+func (d *deleteNLB) Delete(t fi.CloudupTarget) error {
+	ctx := context.TODO()
+
+	awsTarget, ok := t.(*awsup.AWSAPITarget)
+	if !ok {
+		return fmt.Errorf("unexpected target type for deletion: %T", t)
+	}
+
+	arn := d.obj.ARN()
+	klog.V(2).Infof("deleting load balancer %q", arn)
+	if _, err := awsTarget.Cloud.ELBV2().DeleteLoadBalancerWithContext(ctx, &elbv2.DeleteLoadBalancerInput{
+		LoadBalancerArn: &arn,
+	}); err != nil {
+		return fmt.Errorf("error deleting ELB LoadBalancer %q: %w", arn, err)
+	}
+
+	return nil
+}
+
+// String returns a string representation of the task
+func (d *deleteNLB) String() string {
+	return d.TaskName() + "-" + d.Item()
+}
+
+// TaskName returns the task name
+func (d *deleteNLB) TaskName() string {
+	return "network-load-balancer"
+}
+
+// Item returns the launch template name
+func (d *deleteNLB) Item() string {
+	return d.obj.ARN()
+}
+
+func (d *deleteNLB) DeferDeletion() bool {
+	return true
 }
