@@ -282,11 +282,7 @@ func PopulateInstanceGroupSpec(cluster *kops.Cluster, input *kops.InstanceGroup,
 	{
 		if ig.IsControlPlane() {
 			// (Even though the value is empty, we still expect <Key>=<Value>:<Effect>)
-			if cluster.IsKubernetesLT("1.24") {
-				taints.Insert(nodelabels.RoleLabelMaster16 + "=:" + string(v1.TaintEffectNoSchedule))
-			} else {
-				taints.Insert(nodelabels.RoleLabelControlPlane20 + "=:" + string(v1.TaintEffectNoSchedule))
-			}
+			taints.Insert(nodelabels.RoleLabelControlPlane20 + "=:" + string(v1.TaintEffectNoSchedule))
 		}
 		if ig.IsAPIServerOnly() {
 			// (Even though the value is empty, we still expect <Key>=<Value>:<Effect>)
