@@ -32,10 +32,10 @@ import (
 	"time"
 
 	"github.com/aws/aws-sdk-go/aws"
+	"github.com/aws/aws-sdk-go/aws/awserr"
 	"github.com/aws/aws-sdk-go/aws/ec2metadata"
 	"github.com/aws/aws-sdk-go/aws/session"
 	"github.com/aws/aws-sdk-go/service/autoscaling"
-	"github.com/aws/aws-sdk-go/service/ec2"
 	"github.com/aws/aws-sdk-go/service/kms"
 	"go.uber.org/multierr"
 	"k8s.io/klog/v2"
@@ -46,15 +46,14 @@ import (
 	"k8s.io/kops/pkg/apis/nodeup"
 	"k8s.io/kops/pkg/assets"
 	"k8s.io/kops/pkg/bootstrap"
+	"k8s.io/kops/pkg/bootstrap/pkibootstrap"
 	"k8s.io/kops/pkg/configserver"
 	"k8s.io/kops/pkg/kopscontrollerclient"
-	"k8s.io/kops/pkg/resolver"
 	"k8s.io/kops/pkg/wellknownports"
 	"k8s.io/kops/upup/pkg/fi"
 	"k8s.io/kops/upup/pkg/fi/cloudup/awsup"
 	"k8s.io/kops/upup/pkg/fi/cloudup/azure"
 	"k8s.io/kops/upup/pkg/fi/cloudup/do"
-	"k8s.io/kops/upup/pkg/fi/cloudup/gce/gcediscovery"
 	"k8s.io/kops/upup/pkg/fi/cloudup/gce/tpm/gcetpmsigner"
 	"k8s.io/kops/upup/pkg/fi/cloudup/hetzner"
 	"k8s.io/kops/upup/pkg/fi/cloudup/openstack"
@@ -154,8 +153,10 @@ func (c *NodeUpCommand) Run(out io.Writer) error {
 		return fmt.Errorf("no instance group defined in nodeup config")
 	}
 
-	if want, got := bootConfig.NodeupConfigHash, base64.StdEncoding.EncodeToString(nodeupConfigHash[:]); got != want {
-		return fmt.Errorf("nodeup config hash mismatch (was %q, expected %q)", got, want)
+	if bootConfig.NodeupConfigHash != "" {
+		if want, got := bootConfig.NodeupConfigHash, base64.StdEncoding.EncodeToString(nodeupConfigHash[:]); got != want {
+			return fmt.Errorf("nodeup config hash mismatch (was %q, expected %q)", got, want)
+		}
 	}
 
 	err = evaluateSpec(&nodeupConfig, bootConfig.CloudProvider)
@@ -245,9 +246,12 @@ func (c *NodeUpCommand) Run(out io.Writer) error {
 		}
 		modelContext.InstanceID = string(instanceIDBytes)
 
-		modelContext.ConfigurationMode, err = getAWSConfigurationMode(modelContext)
-		if err != nil {
-			return err
+		// Check if WarmPool is enabled first, to avoid additional API calls
+		if len(modelContext.NodeupConfig.WarmPoolImages) > 0 {
+			modelContext.ConfigurationMode, err = getAWSConfigurationMode(ctx, modelContext)
+			if err != nil {
+				return err
+			}
 		}
 
 		modelContext.MachineType, err = getMachineType()
@@ -289,7 +293,6 @@ func (c *NodeUpCommand) Run(out io.Writer) error {
 	loader.Builders = append(loader.Builders, &model.UpdateServiceBuilder{NodeupModelContext: modelContext})
 	loader.Builders = append(loader.Builders, &model.VolumesBuilder{NodeupModelContext: modelContext})
 	loader.Builders = append(loader.Builders, &model.ContainerdBuilder{NodeupModelContext: modelContext})
-	loader.Builders = append(loader.Builders, &model.DockerBuilder{NodeupModelContext: modelContext})
 	loader.Builders = append(loader.Builders, &model.ProtokubeBuilder{NodeupModelContext: modelContext})
 	loader.Builders = append(loader.Builders, &model.CloudConfigBuilder{NodeupModelContext: modelContext})
 	loader.Builders = append(loader.Builders, &model.FileAssetsBuilder{NodeupModelContext: modelContext})
@@ -311,6 +314,8 @@ func (c *NodeUpCommand) Run(out io.Writer) error {
 	loader.Builders = append(loader.Builders, &model.KopsControllerBuilder{NodeupModelContext: modelContext})
 	loader.Builders = append(loader.Builders, &model.WarmPoolBuilder{NodeupModelContext: modelContext})
 	loader.Builders = append(loader.Builders, &model.PrefixBuilder{NodeupModelContext: modelContext})
+	loader.Builders = append(loader.Builders, &model.NerdctlBuilder{NodeupModelContext: modelContext})
+	loader.Builders = append(loader.Builders, &model.CrictlBuilder{NodeupModelContext: modelContext})
 
 	loader.Builders = append(loader.Builders, &networking.CommonBuilder{NodeupModelContext: modelContext})
 	loader.Builders = append(loader.Builders, &networking.CalicoBuilder{NodeupModelContext: modelContext})
@@ -328,7 +333,6 @@ func (c *NodeUpCommand) Run(out io.Writer) error {
 		taskMap["LoadImage."+strconv.Itoa(i)] = &nodetasks.LoadImageTask{
 			Sources: image.Sources,
 			Hash:    image.Hash,
-			Runtime: nodeupConfig.ContainerRuntime,
 		}
 	}
 	// Protokube load image task is in ProtokubeBuilder
@@ -368,7 +372,7 @@ func (c *NodeUpCommand) Run(out io.Writer) error {
 
 	if nodeupConfig.EnableLifecycleHook {
 		if bootConfig.CloudProvider == api.CloudProviderAWS {
-			err := completeWarmingLifecycleAction(cloud.(awsup.AWSCloud), modelContext)
+			err := completeWarmingLifecycleAction(ctx, cloud.(awsup.AWSCloud), modelContext)
 			if err != nil {
 				return fmt.Errorf("failed to complete lifecylce action: %w", err)
 			}
@@ -392,11 +396,11 @@ func getMachineType() (string, error) {
 	return instanceTypeName, err
 }
 
-func completeWarmingLifecycleAction(cloud awsup.AWSCloud, modelContext *model.NodeupModelContext) error {
+func completeWarmingLifecycleAction(ctx context.Context, cloud awsup.AWSCloud, modelContext *model.NodeupModelContext) error {
 	asgName := modelContext.BootConfig.InstanceGroupName + "." + modelContext.NodeupConfig.ClusterName
 	hookName := "kops-warmpool"
 	svc := cloud.Autoscaling()
-	hooks, err := svc.DescribeLifecycleHooks(&autoscaling.DescribeLifecycleHooksInput{
+	hooks, err := svc.DescribeLifecycleHooksWithContext(ctx, &autoscaling.DescribeLifecycleHooksInput{
 		AutoScalingGroupName: &asgName,
 		LifecycleHookNames:   []*string{&hookName},
 	})
@@ -406,7 +410,7 @@ func completeWarmingLifecycleAction(cloud awsup.AWSCloud, modelContext *model.No
 
 	if len(hooks.LifecycleHooks) > 0 {
 		klog.Info("Found ASG lifecycle hook")
-		_, err := svc.CompleteLifecycleAction(&autoscaling.CompleteLifecycleActionInput{
+		_, err := svc.CompleteLifecycleActionWithContext(ctx, &autoscaling.CompleteLifecycleActionInput{
 			AutoScalingGroupName:  &asgName,
 			InstanceId:            &modelContext.InstanceID,
 			LifecycleHookName:     &hookName,
@@ -423,7 +427,7 @@ func completeWarmingLifecycleAction(cloud awsup.AWSCloud, modelContext *model.No
 }
 
 func evaluateSpec(nodeupConfig *nodeup.Config, cloudProvider api.CloudProviderID) error {
-	hostnameOverride, err := evaluateHostnameOverride(cloudProvider, nodeupConfig.UseInstanceIDForNodeName)
+	hostnameOverride, err := evaluateHostnameOverride(cloudProvider)
 	if err != nil {
 		return err
 	}
@@ -438,59 +442,18 @@ func evaluateSpec(nodeupConfig *nodeup.Config, cloudProvider api.CloudProviderID
 		}
 	}
 
-	if nodeupConfig.ContainerRuntime == "docker" {
-		err = evaluateDockerSpecStorage(nodeupConfig.Docker)
-		if err != nil {
-			return err
-		}
-	}
-
 	return nil
 }
 
-func evaluateHostnameOverride(cloudProvider api.CloudProviderID, useInstanceIDForNodeName bool) (string, error) {
+func evaluateHostnameOverride(cloudProvider api.CloudProviderID) (string, error) {
 	switch cloudProvider {
 	case api.CloudProviderAWS:
 		instanceIDBytes, err := vfs.Context.ReadFile("metadata://aws/meta-data/instance-id")
 		if err != nil {
 			return "", fmt.Errorf("error reading instance-id from AWS metadata: %v", err)
 		}
-		instanceID := string(instanceIDBytes)
 
-		if useInstanceIDForNodeName {
-			return instanceID, nil
-		}
-
-		azBytes, err := vfs.Context.ReadFile("metadata://aws/meta-data/placement/availability-zone")
-		if err != nil {
-			return "", fmt.Errorf("error reading availability zone from AWS metadata: %v", err)
-		}
-
-		config := aws.NewConfig()
-		config = config.WithCredentialsChainVerboseErrors(true)
-
-		s, err := session.NewSession(config)
-		if err != nil {
-			return "", fmt.Errorf("error starting new AWS session: %v", err)
-		}
-
-		svc := ec2.New(s, config.WithRegion(string(azBytes[:len(azBytes)-1])))
-
-		result, err := svc.DescribeInstances(&ec2.DescribeInstancesInput{
-			InstanceIds: []*string{&instanceID},
-		})
-		if err != nil {
-			return "", fmt.Errorf("error describing instances: %v", err)
-		}
-
-		if len(result.Reservations) > 1 {
-			return "", fmt.Errorf("too many reservations returned for the single instance-id")
-		}
-		if len(result.Reservations[0].Instances) > 1 {
-			return "", fmt.Errorf("too many instances returned for the single instance-id")
-		}
-
-		return *(result.Reservations[0].Instances[0].PrivateDnsName), nil
+		return string(instanceIDBytes), nil
 
 	case api.CloudProviderGCE:
 		// This lets us tolerate broken hostnames (i.e. systemd)
@@ -551,56 +514,6 @@ func evaluateBindAddress(bindAddress string) (string, error) {
 	return bindAddress, nil
 }
 
-// evaluateDockerSpec selects the first supported storage mode, if it is a list
-func evaluateDockerSpecStorage(spec *api.DockerConfig) error {
-	storage := fi.ValueOf(spec.Storage)
-	if strings.Contains(fi.ValueOf(spec.Storage), ",") {
-		precedence := strings.Split(storage, ",")
-		for _, opt := range precedence {
-			fs := opt
-			if fs == "overlay2" {
-				fs = "overlay"
-			}
-			supported, err := kernelHasFilesystem(fs)
-			if err != nil {
-				klog.Warningf("error checking if %q filesystem is supported: %v", fs, err)
-				continue
-			}
-
-			if !supported {
-				// overlay -> overlay
-				// aufs -> aufs
-				module := fs
-				if err = modprobe(fs); err != nil {
-					klog.Warningf("error running `modprobe %q`: %v", module, err)
-				}
-			}
-
-			supported, err = kernelHasFilesystem(fs)
-			if err != nil {
-				klog.Warningf("error checking if %q filesystem is supported: %v", fs, err)
-				continue
-			}
-
-			if supported {
-				klog.Infof("Using supported docker storage %q", opt)
-				spec.Storage = fi.PtrTo(opt)
-				return nil
-			}
-
-			klog.Warningf("%q docker storage was specified, but filesystem is not supported", opt)
-		}
-
-		// Just in case we don't recognize the driver?
-		// TODO: Is this the best behaviour
-		klog.Warningf("No storage module was supported from %q, will default to %q", storage, precedence[0])
-		spec.Storage = fi.PtrTo(precedence[0])
-		return nil
-	}
-
-	return nil
-}
-
 // kernelHasFilesystem checks if /proc/filesystems contains the specified filesystem
 func kernelHasFilesystem(fs string) (bool, error) {
 	contents, err := os.ReadFile("/proc/filesystems")
@@ -698,7 +611,6 @@ func seedRNG(ctx context.Context, bootConfig *nodeup.BootConfig, region string)
 // getNodeConfigFromServers queries kops-controllers for our node's configuration.
 func getNodeConfigFromServers(ctx context.Context, bootConfig *nodeup.BootConfig, region string) (*nodeup.BootstrapResponse, error) {
 	var authenticator bootstrap.Authenticator
-	var resolver resolver.Resolver
 
 	switch bootConfig.CloudProvider {
 	case api.CloudProviderAWS:
@@ -713,12 +625,6 @@ func getNodeConfigFromServers(ctx context.Context, bootConfig *nodeup.BootConfig
 			return nil, err
 		}
 		authenticator = a
-
-		discovery, err := gcediscovery.New()
-		if err != nil {
-			return nil, err
-		}
-		resolver = discovery
 	case api.CloudProviderHetzner:
 		a, err := hetzner.NewHetznerAuthenticator()
 		if err != nil {
@@ -749,6 +655,14 @@ func getNodeConfigFromServers(ctx context.Context, bootConfig *nodeup.BootConfig
 			return nil, err
 		}
 		authenticator = a
+
+	case "metal":
+		a, err := pkibootstrap.NewAuthenticatorFromFile("/etc/kubernetes/kops/pki/machine/private.pem")
+		if err != nil {
+			return nil, err
+		}
+		authenticator = a
+
 	default:
 		return nil, fmt.Errorf("unsupported cloud provider for node configuration %s", bootConfig.CloudProvider)
 	}
@@ -772,7 +686,6 @@ func getNodeConfigFromServers(ctx context.Context, bootConfig *nodeup.BootConfig
 
 	client := &kopscontrollerclient.Client{
 		Authenticator: authenticator,
-		Resolver:      resolver,
 		CAs:           []byte(bootConfig.ConfigServer.CACertificates),
 	}
 
@@ -805,7 +718,12 @@ func getNodeConfigFromServers(ctx context.Context, bootConfig *nodeup.BootConfig
 	return nil, merr
 }
 
-func getAWSConfigurationMode(c *model.NodeupModelContext) (string, error) {
+func getAWSConfigurationMode(ctx context.Context, c *model.NodeupModelContext) (string, error) {
+	// Check if WarmPool is enabled first, to avoid additional API calls
+	if len(c.NodeupConfig.WarmPoolImages) == 0 {
+		return "", nil
+	}
+
 	// Only worker nodes and apiservers can actually autoscale.
 	// We are not adding describe permissions to the other roles
 	role := c.BootConfig.InstanceGroupRole
@@ -813,20 +731,17 @@ func getAWSConfigurationMode(c *model.NodeupModelContext) (string, error) {
 		return "", nil
 	}
 
-	svc := c.Cloud.(awsup.AWSCloud).Autoscaling()
-
-	result, err := svc.DescribeAutoScalingInstances(&autoscaling.DescribeAutoScalingInstancesInput{
-		InstanceIds: []*string{&c.InstanceID},
-	})
+	targetLifecycleState, err := vfs.Context.ReadFile("metadata://aws/meta-data/autoscaling/target-lifecycle-state")
 	if err != nil {
-		return "", fmt.Errorf("error describing instances: %v", err)
+		var awsErr awserr.RequestFailure
+		if errors.As(err, &awsErr) && awsErr.StatusCode() == 404 {
+			// The instance isn't in an ASG (karpenter, etc.)
+			return "", nil
+		}
+		return "", fmt.Errorf("error reading target-lifecycle-state from instance metadata: %v", err)
 	}
-	// If the instance is not a part of an ASG, it won't be in a warm pool either.
-	if len(result.AutoScalingInstances) < 1 {
-		return "", nil
-	}
-	lifecycle := fi.ValueOf(result.AutoScalingInstances[0].LifecycleState)
-	if strings.HasPrefix(lifecycle, "Warmed:") {
+
+	if strings.HasPrefix(string(targetLifecycleState), "Warmed:") {
 		klog.Info("instance is entering warm pool")
 		return model.ConfigurationModeWarming, nil
 	} else {
